

class Instrument:
    BTC = 'BTC'
    ETH = 'ETH'
    LTC = 'LTC'
    ETC = 'ETC'


class Currency:
    AUD = 'AUD'
    BTC = 'BTC'


class OrderSide:
    Buy = 'Bid'
    Sell = 'Ask'


class OrderType:
    LIMIT = 'Limit'
    MARKET = 'Market'


class Multipliers:
    PRICE = 100000000
    VOLUME = 100000000



